require 'test/unit'
require File.dirname(__FILE__) + '/../classifier.rb'

class TestOfClassifier < Test::Unit::TestCase

    def test_after_learning_single_document_then_the_same_document_matches_exactly
        classifier = Classifier.new
        classifier.learn( 'stuff', 'Hello world' )
        assert_equal(
                { 'stuff' => 1.0 },
                classifier.distribution( 'Hello world' ))
    end

    def test_a_document_classified_in_two_categories_will_split_a_match_for_the_same
        classifier = Classifier.new
        classifier.learn( 'A', 'Hello world' )
        classifier.learn( 'B', 'Hello world' )
        assert_equal(
            { 'A' => 0.5, 'B' => 0.5 },
            classifier.distribution( 'Hello world' ))
    end
  
    def test_with_only_one_word_in_each_category_a_single_word_discriminates_perfectly
        classifier = Classifier.new
        classifier.learn( 'A', 'Apple' )
        classifier.learn( 'B', 'Banana' )
        classifier.learn( 'C', 'Cherry' )
        assert_equal( ['B'], classifier.classify( 'Banana' ))
    end
  
    def test_document_with_words_from_two_categories_is_equally_likkely_to_be_in_both
        classifier = Classifier.new
        classifier.learn( 'A', 'Apple' )
        classifier.learn( 'B', 'Banana' )
        classifier.learn( 'C', 'Cherry' )
        assert_equal(
                ['B', 'C'],
                classifier.classify( 'Cherry Banana' ))
    end
  
    def test_document_with_one_word_from_every_category_is_equally_likely_to_be_in_any
        classifier = Classifier.new
        classifier.learn( 'A', 'Apple' )
        classifier.learn( 'B', 'Banana' )
        classifier.learn( 'C', 'Cherry' )
        assert_equal(
                ['A', 'B', 'C'],
                classifier.classify( 'Cherry Banana Apple' ))
    end
  
    def test_more_hits_on_a_word_in_oe_category_promotes_that_category
        classifier = Classifier.new
        classifier.learn( 'A', 'Apple' )
        classifier.learn( 'B', 'Banana' )
        classifier.learn( 'C', 'Cherry' )
        assert_equal(
                ['C'],
                classifier.classify( 'Cherry Banana Cherry Apple Cherry' ))
    end
  
    def test_it_takes_a_significant_preference_for_words_in_one_category_to_discriminate
        classifier = Classifier.new
        classifier.learn( 'A', 'a' )
        classifier.learn( 'B', 'b' )
        classifier.learn( 'C', 'c' )
        assert_equal(
                ['A', 'B', 'C'],
                classifier.classify( 'a b c a b c a' ))
    end
  
    def test_multiword_documents_are_weaker_than_single_word_documents
        classifier = Classifier.new
        classifier.learn( 'A', 'Apple' )
        classifier.learn( 'B', 'Banana Apple Mango' )
        assert_equal( ['A'], classifier.classify( 'Apple' ))
    end
  
    def test_multiword_searches_with_more_hits_win
        classifier = Classifier.new
        classifier.learn( 'A', 'Apple Mango Cherry' )
        classifier.learn( 'B', 'Banana Apple Banana Mango' )
        assert_equal( ['B'], classifier.classify( 'Apple Banana' ))
    end
end
