require 'test/unit'
require File.dirname(__FILE__) + '/../probabilities.rb'

class TestOfProbabilites < Test::Unit::TestCase

    def testNewCategoryStartsAtOne
        probabilities = ProbabilityVector.new
        probabilities.multiply( 'a', 1.0 )
        assert_equal( { 'a' => 1.0 }, probabilities.normalised )
        assert_equal(
                ['a'],
                probabilities.likely { |score, highest| score > 0.5 * highest } )
    end
  
    def testSingleValueAlwaysNormalisedToOne
        probabilities = ProbabilityVector.new
        probabilities.multiply( 'a', 2.0 )
        assert_equal( { 'a' => 1.0 }, probabilities.normalised )
        assert_equal(
                ['a'],
                probabilities.likely { |score, highest| score > 0.5 * highest } )
    end
  
    def testEasyVictoryForFirstEntry
        probabilities = ProbabilityVector.new
        probabilities.multiply( 'a', 9.0 )
        probabilities.multiply( 'b', 1.0 )
        assert_equal( { 'a' => 0.9, 'b' => 0.1 }, probabilities.normalised )
        assert_equal(
                ['a'],
                probabilities.likely { |score, highest| score > 0.5 * highest } )
    end
  
    def testEasyVictoryForMiddleEntry
        probabilities = ProbabilityVector.new
        probabilities.multiply( 'a', 1.0 )
        probabilities.multiply( 'b', 8.0 )
        probabilities.multiply( 'c', 1.0 )
        assert_equal(
                { 'a' => 0.1, 'b' => 0.8, 'c' => 0.1 },
                probabilities.normalised )
        assert_equal(
                ['b'],
                probabilities.likely { |score, highest| score > 0.5 * highest } )
    end
    
    def testJointVictoryForEqualEntries
        probabilities = ProbabilityVector.new
        probabilities.multiply( 'a', 1.0 )
        probabilities.multiply( 'b', 1.0 )
        assert_equal( { 'a' => 0.5, 'b' => 0.5 }, probabilities.normalised )
        assert_equal(
                ['a', 'b'],
                probabilities.likely { |score, highest| score > 0.5 * highest } )
    end
    
    def testJointVictoryForNearEntries
        probabilities = ProbabilityVector.new
        probabilities.multiply( 'a', 3.0 )
        probabilities.multiply( 'b', 2.0 )
        assert_equal( { 'a' => 0.6, 'b' => 0.4 }, probabilities.normalised )
        assert_equal(
                ['a', 'b'],
                probabilities.likely { |score, highest| score > 0.5 * highest } )
    end
  
    def testJointVictoryForNearEntriesInGroup
        probabilities = ProbabilityVector.new
        probabilities.multiply( 'a', 1.0 )
        probabilities.multiply( 'b', 2.0 )
        probabilities.multiply( 'c', 3.0 )
        probabilities.multiply( 'd', 4.0 )
        assert_equal(
                { 'a' => 0.1, 'b' => 0.2, 'c' => 0.3, 'd' => 0.4 },
                probabilities.normalised )
        assert_equal(
                ['c', 'd'],
                probabilities.likely { |score, highest| score > 0.5 * highest } )
    end
end
